package com.link.core.mq.cmq.Tools;

import com.link.core.mq.cmq.Tools.Json.JSONArray;
import com.link.core.mq.cmq.Tools.Json.JSONObject;
import java.util.List;
import java.util.TreeMap;

public class Account {
   protected CMQClient client;

   public Account(String endpoint, String secretId, String secretKey) {
      this.client = new CMQClient(endpoint, "/v2/index.php", secretId, secretKey, "POST");
   }

   public Account(String secretId, String secretKey, String endpoint, String path, String method) {
      this.client = new CMQClient(endpoint, path, secretId, secretKey, method);
   }

   public void setSignMethod(String Method) {
      this.client.setSignMethod(Method);
   }

   public void createQueue(String queueName, QueueMeta meta) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if ("".equals(queueName)) {
         throw new CMQClientException("Invalid parameter:queueName is empty");
      } else {
         param.put("queueName", queueName);
         if (meta.getMaxMsgHeapNum() > 0) {
            param.put("maxMsgHeapNum", Integer.toString(meta.getMaxMsgHeapNum()));
         }

         if (meta.getPollingWaitSeconds() > 0) {
            param.put("pollingWaitSeconds", Integer.toString(meta.getPollingWaitSeconds()));
         }

         if (meta.getVisibilityTimeout() > 0) {
            param.put("visibilityTimeout", Integer.toString(meta.getVisibilityTimeout()));
         }

         if (meta.getMaxMsgSize() > 0) {
            param.put("maxMsgSize", Integer.toString(meta.getMaxMsgSize()));
         }

         if (meta.getMsgRetentionSeconds() > 0) {
            param.put("msgRetentionSeconds", Integer.toString(meta.getMsgRetentionSeconds()));
         }

         String result = this.client.call("CreateQueue", param);
         JSONObject jsonObj = new JSONObject(result);
         int code = jsonObj.getInt("code");
         if (code != 0) {
            throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
         }
      }
   }

   public void deleteQueue(String queueName) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if ("".equals(queueName)) {
         throw new CMQClientException("Invalid parameter:queueName is empty");
      } else {
         param.put("queueName", queueName);
         String result = this.client.call("DeleteQueue", param);
         JSONObject jsonObj = new JSONObject(result);
         int code = jsonObj.getInt("code");
         if (code != 0) {
            throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
         }
      }
   }

   public int listQueue(String searchWord, int offset, int limit, List<String> queueList) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if (!"".equals(searchWord)) {
         param.put("searchWord", searchWord);
      }

      if (offset >= 0) {
         param.put("offset", Integer.toString(offset));
      }

      if (limit > 0) {
         param.put("limit", Integer.toString(limit));
      }

      String result = this.client.call("ListQueue", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         int totalCount = jsonObj.getInt("totalCount");
         JSONArray jsonArray = jsonObj.getJSONArray("queueList");

         for(int i = 0; i < jsonArray.length(); ++i) {
            JSONObject obj = (JSONObject)jsonArray.get(i);
            queueList.add(obj.getString("queueName"));
         }

         return totalCount;
      }
   }

   public Queue getQueue(String queueName) {
      return new Queue(queueName, this.client);
   }

   public Topic getTopic(String topicName) {
      return new Topic(topicName, this.client);
   }

   public void createTopic(final String topicName, final int maxMsgSize) throws Exception {
      this.createTopic(topicName, maxMsgSize, 1);
   }

   public void createTopic(final String topicName, final int maxMsgSize, int filterType) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if ("".equals(topicName)) {
         throw new CMQClientException("Invalid parameter:topicName is empty");
      } else {
         param.put("topicName", topicName);
         param.put("filterType", Integer.toString(filterType));
         if (maxMsgSize >= 1 && maxMsgSize <= 65536) {
            param.put("maxMsgSize", Integer.toString(maxMsgSize));
            String result = this.client.call("CreateTopic", param);
            JSONObject jsonObj = new JSONObject(result);
            int code = jsonObj.getInt("code");
            if (code != 0) {
               throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
            }
         } else {
            throw new CMQClientException("Invalid parameter: maxMsgSize > 65536 or maxMsgSize < 1");
         }
      }
   }

   public void deleteTopic(final String topicName) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if ("".equals(topicName)) {
         throw new CMQClientException("Invalid parameter:topicName is empty");
      } else {
         param.put("topicName", topicName);
         String result = this.client.call("DeleteTopic", param);
         JSONObject jsonObj = new JSONObject(result);
         int code = jsonObj.getInt("code");
         if (code != 0) {
            throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
         }
      }
   }

   public int listTopic(final String searchWord, List<String> vTopicList, final int offset, final int limit) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if (!"".equals(searchWord)) {
         param.put("searchWord", searchWord);
      }

      if (offset >= 0) {
         param.put("offset", Integer.toString(offset));
      }

      if (limit > 0) {
         param.put("limit", Integer.toString(limit));
      }

      String result = this.client.call("ListTopic", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         int totalCount = jsonObj.getInt("totalCount");
         JSONArray jsonArray = jsonObj.getJSONArray("topicList");
         vTopicList.clear();

         for(int i = 0; i < jsonArray.length(); ++i) {
            JSONObject obj = (JSONObject)jsonArray.get(i);
            vTopicList.add(obj.getString("topicName"));
         }

         return totalCount;
      }
   }

   public void createSubscribe(final String topicName, final String subscriptionName, final String Endpoint, final String Protocal) throws Exception {
      this.createSubscribe(topicName, subscriptionName, Endpoint, Protocal, (List)null, (List)null, "BACKOFF_RETRY", "JSON");
   }

   public void createSubscribe(final String topicName, final String subscriptionName, final String Endpoint, final String Protocal, final List<String> FilterTag, final List<String> bindingKey, final String NotifyStrategy, final String NotifyContentFormat) throws Exception {
      if (FilterTag != null && FilterTag.size() > 5) {
         throw new CMQClientException("Invalid parameter: Tag number > 5");
      } else {
         TreeMap<String, String> param = new TreeMap();
         if ("".equals(topicName)) {
            throw new CMQClientException("Invalid parameter:topicName is empty");
         } else {
            param.put("topicName", topicName);
            if ("".equals(subscriptionName)) {
               throw new CMQClientException("Invalid parameter:subscriptionName is empty");
            } else {
               param.put("subscriptionName", subscriptionName);
               if ("".equals(Endpoint)) {
                  throw new CMQClientException("Invalid parameter:Endpoint is empty");
               } else {
                  param.put("endpoint", Endpoint);
                  if ("".equals(Protocal)) {
                     throw new CMQClientException("Invalid parameter:Protocal is empty");
                  } else {
                     param.put("protocol", Protocal);
                     if ("".equals(NotifyStrategy)) {
                        throw new CMQClientException("Invalid parameter:NotifyStrategy is empty");
                     } else {
                        param.put("notifyStrategy", NotifyStrategy);
                        if ("".equals(NotifyContentFormat)) {
                           throw new CMQClientException("Invalid parameter:NotifyContentFormat is empty");
                        } else {
                           param.put("notifyContentFormat", NotifyContentFormat);
                           int i;
                           if (FilterTag != null) {
                              for(i = 0; i < FilterTag.size(); ++i) {
                                 param.put("filterTag." + Integer.toString(i + 1), FilterTag.get(i));
                              }
                           }

                           if (bindingKey != null) {
                              for(i = 0; i < bindingKey.size(); ++i) {
                                 param.put("bindingKey." + Integer.toString(i + 1), bindingKey.get(i));
                              }
                           }

                           String result = this.client.call("Subscribe", param);
                           JSONObject jsonObj = new JSONObject(result);
                           int code = jsonObj.getInt("code");
                           if (code != 0) {
                              throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   public void deleteSubscribe(final String topicName, final String subscriptionName) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      if ("".equals(topicName)) {
         throw new CMQClientException("Invalid parameter:topicName is empty");
      } else {
         param.put("topicName", topicName);
         if ("".equals(subscriptionName)) {
            throw new CMQClientException("Invalid parameter:subscriptionName is empty");
         } else {
            param.put("subscriptionName", subscriptionName);
            String result = this.client.call("Unsubscribe", param);
            JSONObject jsonObj = new JSONObject(result);
            int code = jsonObj.getInt("code");
            if (code != 0) {
               throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
            }
         }
      }
   }

   public Subscription getSubscription(final String topicName, final String subscriptionName) {
      return new Subscription(topicName, subscriptionName, this.client);
   }
}
